; ------------------------------------------------------------------------------------------------------------------------------------------
TIMER		equ	$0113							; IRQ managed 1 byte timer
; ------------------------------------------------------------------------------------------------------------------------------------------
			org	$0e00							; make it CoCo compatible
Start		orcc	#$50							; disable interrupts
NewGame	lds	#$03d6						; set stack to end of Buffer for List
; ------------------------------------------------------------------------------------------------------------------------------------------
; Main game engine code starts here
; ------------------------------------------------------------------------------------------------------------------------------------------
Game		jsr	CfgHdw						; config Sound and Interrupts
; ------------------------------------------------------------------------------------------------------------------------------------------
			bsr	SetScr
MLoop		jsr	VSync							; synchronize with blank interrupt
			lbsr	ReadAll
			ldu	#$506
			ldx	#txt3
			bsr	ShowStr
			dec	vByte
			bne	1f
			lda 	#50
			sta	sndTyp
1			jsr	ShowData						; and show values 
			bra	MLoop							; loopback
; ------------------------------------------------------------------------------------------------------------------------------------------
ShowData	
ShowDEx	rts
; ==========================================================================================================================================
SetScr	ldx	#$400
			ldd	#$cfcf
1			std	,x++
			cmpx	#$600
			bne	1b
			ldu	#$4c6
			ldx	#txt1
			bsr	ShowStr
			ldu	#$4e6
			ldx	#txt2
			bsr	ShowStr
			rts
; ------------------------------------------------------------------------------------------------------------------------------------------
ShowStr	lda	,x+
			beq	ShowSEx
			ora	#64
			sta	,u+
			bra	ShowStr
ShowSEx	rts
; ------------------------------------------------------------------------------------------------------------------------------------------
txt1		fcn	/        FIRE  OPTION  /
txt2		fcn	/R L D U 1 2 3 1 2 3 4 /
txt3		fill	$cf,14						; space for the 4 moves and 3 fires
txt4		fill	$cf,7							; space for the 4 options
txt5		fcb	$00
; ------------------------------------------------------------------------------------------------------------------------------------------
ReadAll	jsr 	JoyKey 						; read Joystick (LRUPFFF)
			ldu	#txt3
			lda	joyVal
2			ldb	#$cf							; no pressed
			rora
			bcc	1f
			ldb	#143+48						; pressed
1			stb	,u++
			cmpu	#txt4
			bne	2b
												; now read the options
			ldx	#keys+7						; point to option 1
4			lda	,x								; get mask, point to next
			jsr	KTest							; test key pressed
			bcs	3f								; yes skip two
			ldb	#$cf							; get not pressed
			bra	5f								; show it
3			ldb	#143+48						; get pressed
5			stb	,u++							; show it and point to next
			leax	1,x
			cmpx	#keys+11						; got to the end?
			blo	4b								; no, loop back for next
			rts
; ------------------------------------------------------------------------------------------------------------------------------------------
; config DAC sound and VSync detection at the end of frame
; ------------------------------------------------------------------------------------------------------------------------------------------
CfgHdw	clrb									; flag for Dragon machine
			lda	$fffe							; read high byte for Reset address
			cmpa	#$b3							; is it a Dragon? ($b3b4)
			beq	CfgHdw1						; yes, skip next
			incb									; set flag for CoCo machine
CfgHdw1	stb	cocoFlg						; write to variable
			ldb	#keysD-keys					; get offset to Dragon codes
			tst	cocoFlg						; is this a CoCo?
			beq	CfgNoCC						; no, skip next
			ldb	#keysC-keys					; get offset to CoCo codes
CfgNoCC	stb	scanOff						; offset for scanning keys / joysticks
			lda	$ff01							; get config PIA0-SideA
			anda	#%11110100					; reset LSB of MUX. Disable HS IRQ to CPU
			sta	$ff01							; update config PIA0-SideA
			lda	$ff03							; get config PIA0-SideB
			anda	#%11110100					; reset MSB of MUX -> %00 select DAC sound. Flag set on falling edge
			ora	#%00000001					; Enable FS IRQ to CPU
			sta	$ff03							; update config PIA0-SideB
			clr	$ff20							; set zero sound volume
			lda	$ff23							; get config PIA1-SideB
			ora	#%00001000					; enable DAC sound
			sta	$ff23							; update config PIA1-SideB
			ldx	#IRQHndlr					; get new IRQ dispatcher address
			stx	$010d							; set in system hook
			andcc	#$ef							; allow IRQ			
			rts									; return
; ------------------------------------------------------------------------------------------------------------------------------------------
; new IRQ Handler (for Vertical Blanking interrupt)
; ------------------------------------------------------------------------------------------------------------------------------------------
IRQHndlr	lda	$ff03							; is an FS IRQ?
			bmi	IRQRcvd						; yes, skip next
			rti									; return from interrupt
IRQRcvd	lda	$ff02							; clear flag
			inc	TIMER							; increment counter
			rti									; return from interrupt
; ------------------------------------------------------------------------------------------------------------------------------------------
; Screen synchronisation.																			
;  - read joystick/keyboard																		
;  - handle sound																						
;  - sync framerate with clock																	
;  - handle shrapnel																					
; -------------------------------------------------------------------------------
VSync		jsr 	JoyKey						; read joystick/keyboard						
			lda	sndTyp						; get sound type									
			bmi	VSync6						; if negative go play white noise			
			beq	VSync1						; if none, exit sound section					
			sta	reg_B						; save as outer loop								
			clra									; get min value for DAC (1st value)			
VSync2	sta	$ff20							; send to DAC										
			eora	#%11111100					; invert value										
			ldb	reg_B						; get loop counter								
			stb	reg_D						; save in reg_D									
VSync3	ldb	TIMER							; get current clock								
			cmpb	clock						; compare to previous clock value			
			blo	VSync4						; if clock is greater exit loop (never)	
			dec	reg_B						; decrement inner loop							
			bne	VSync3						; not zero? loopback								
			ldb	reg_D						; get outer loop counter
			stb	reg_B						; restore in reg_B								
 			dec	reg_B						; decrment outer loop
 			bne	VSync2						; not zero? continue sound						
VSync4	lda	reg_D						; get last used outer counter					
VSynca	sta	sndTyp						; save for next time								
VSync1	jsr	VSync5						; go for shrapnel									
VSync0	lda	TIMER							; get actual timer								
			cmpa	clock						; compare to last read							
			beq	VSync0						; yes, wait until clock changes				
			sta	clock						; update last read value						
VSync5	rts									; return												
; -------------------------------------------------------------------------------
; Play white noise.																					
; -------------------------------------------------------------------------------
VSync6	pshs	b								; save parameter									
			subb	#127							; convert to positive							
VSync7	jsr	Random						; get a random number							
			anda	#%11111100					; clear unused 2 low bits
			sta	$ff20							; send to DAC										
			lda	TIMER							; get actual timer								
			cmpa	clock						; same than last one								
			bne	VSync8						; no, exit loop									
			lda	,s								; get parameter again							
			anda	#127							; reset bit 7 (negative)						
			inca									; increment it										
VSync9	deca									; decrement it										
			bne	VSync9						; not yet zero? loopback						
			dec	,s								; decrement parameter in stack
			bne	VSync7						; not yet zero? loopback						
VSync8	leas	1,s							; clean stack										
			clra									; get value zero (not sound)					
			bra	VSynca						; exit via VSynca									
; -------------------------------------------------------------------------------
; Joystick and keyboard reading routines (Joystick if regA=1, else Keyboard) 		
; -------------------------------------------------------------------------------
JoyKey	lda	contrl						; get kind of control							
			deca									; is it 1?											
			beq	JoyJoy						; yes, process Joystick							
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Keyboard controls.																					
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
			ldb	#7								; number of keys to be read
			pshs	b								; pshs counter
			ldx	#keys+6+1					; point to last key in table					
JoyNoCC	clr	reg_E						; reset bits in output register				
JoyKe0	lda	,-x							; get value from table							
			lbsr	KTest																					
		 	rol	reg_E						; shift into output register					
			dec	,s								; done all keys?									
			bne	JoyKe0						; no, loopback										
			leas	1,s							; get rid of counter
			lbra	JoyJo1						; go store value									
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; from this version ON, this table is added by the compiler at the end of the user code
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
;					 P	  O	A	 Q	 spc  Z	 I	  1	2	 3	  4
keys		fcb	$fe,$7f,$fd,$fd,$7f,$fb,$fd,$fd,$fb,$f7,$ef		; mask to the keyboard columns (only one zero)
keysD		fcb	$10,$08,$04,$10,$20,$20,$08,$01,$01,$01,$01		; expected bit to be zero (Dragon)
keysC		fcb	$04,$02,$01,$04,$08,$08,$02,$10,$10,$10,$10		; expected bit to be zero (CoCo)
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; Joystick controls	- special for 6809 machines (using 6 bit DAC)
; besides the 2 axis (UD, LR) and button, KEYS 'Z' and 'I' must be checked
; defines three zones: UP or LEFT  	 (00-07) - 12,5%
;							  NEUTRAL    	 (08-55) - 75%
;							  DOWN or RIGHT (56-63) - 12,5% 
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
												; fills variables $015c-$015d (X - Y axis left joystick) 
JoyJoy	lbsr	AudiOff						; turn off audio
			leas	-3,s							; 3 bytes for counters
			ldx	#$015e						; point to end of left joystick table
			ldb	#$03							; axis to read -1 (left joystick)
RdJyAx	lda	#$0a							; max number of tries
			std	1,s							; save counters
			bsr	MuxBits						; set both bits
RdJyDo	ldd	#$4080						; $80 first value to DAC, $40 increment
RdJy00	sta	,s								; save 'step'
			orb	#2								; keep RS232 serial out marking
			stb	$ff20							; send 'start' to DAC
			eorb	#2								; put RS232 back to zero
			tst	$ff00							; read data from joystick
			bmi	RdJy01						; data sent too big?
			subb	,s								; deduct step value
			fcb	$8c							; skip next
RdJy01	addb	,s								; add step
			lsra									; logarithmic search (or binary search?)
			cmpa	#1								; lowest step value?
			bne	RdJy00						; not yet
			lsrb									; value in b2-b7
			lsrb									; to b0-b5
			cmpb	-1,x							; compare to last value
			beq	RdSame						; same value twice
			dec	1,s							; debouncing
			bne	RdJyDo						; redetermine
RdSame	stb	,-x							; save value in table
			ldb	2,s							; number of axis
			decb									; decrement it
			bpl	RdJyAx						; next axis
			clr	$ff20							; avoid sound spikes
			bsr	AudiOn						; set audio on again
			leas	3,s							; clean stack
			clr	reg_E						; result to zero
												; process special keys 'I','Z'
			ldx	#keys+6						; point to values table
			lda	,x								; get column value for 'I' (64)
			bsr	KTest							; test for it
		 	rol	reg_E						; shift into output register
		 	lda	,-x							; get colun value for 'Z' (32)
			bsr	KTest							; test for it
		 	rol	reg_E						; shift into output register
		 										; process button 
			andcc	#%11111110					; clear carry
			lda	$ff00							; is button of left Joystick pressed (16)?
			bita	#2								; bit1 is the value (0=YES)
			bne	PutButt						; not pressed
			orcc	#%00000001					; set carry (button is pressed)
PutButt	rol	reg_E						; shift into output register
												; process 2 axis
JoyAxis	lda	>$015d						; get Y axis value
			cmpa	#8								; is it move Up? (now less that 18) was 26
			rol	reg_E						; puts 1 if it was up (08)
			lda	#55							; is it Down? (now greater than 45) was 37
			cmpa	>$015d						; compare to Y axis
			rol	reg_E						; puts 1 if it was down (04)
			lda	>$015c						; get X axis value
			cmpa	#8								; is it move Left? (now less than 18) was 26
			rol	reg_E						; puts 1 if it was right (02)
			lda	#55							; is it move Right? (now greater than 45) was 37
			cmpa	>$015c						; compare to X axis
			rol	reg_E						; puts 1 if it was right (01)
JoyJo1	lda	reg_E						; get result							
JoyJo2	sta	joyVal						; save into variable								
			rts									; return												
; ------------------------------------------------------------------------------
; INPUT: two lower bits in regB -> mutiplex bits									
; ------------------------------------------------------------------------------
MuxBits	ldu	#$ff01						; point to PIA0 CRA
			bsr	MuxB01						; set/reset CA2
MuxB01	lda	,u								; get value from CRA - CRB
			anda	#$f7							; set to zero CA2 - CB2
			rorb									; send bit to carry
			bcc	MuxB02						; not set?
			ora	#8								; set to 1 CA2 - CB2
MuxB02	sta	,u++							; save value, point to CRB
			rts									; return
; ------------------------------------------------------------------------------
AudiOff	clra									; bit 3 of ACCA = 0 disable analog MUX
			fcb	$8c							; skip next
AudiOn	lda	#8								; bit 3 of ACA = 1 enable analog MUX
			sta	,-s							; save on stack
			lda	$ff23							; get ctrl register of PIA1 side-B
			anda	#$f7							; reset bit 3
			ora	,s+							; set desired status
			sta	$ff23							; update PIA
			rts
; ------------------------------------------------------------------------------
; Keyboard test routine. 																			
; -------------------------------------------------------------------------------
KTest		sta	$ff02							; send to keyboard								
			lda	$ff00							; read rows											
			ldb	scanOff						; get offset to codes							
			bita	b,x							; is that bit zero (key pressed)?			
			bne	ScanK01						; no, clear carry									
			orcc	#%00000001					; yes, set carry									
			rts									; return												
ScanK01	andcc	#%11111110					; reset carry flag								
			rts									; return												
; ------------------------------------------------------------------------------
Random	lda	seed
			beq	doEor
			lsla
			beq	noEor
			bcc	noEor
doEor		eora	#$1d
noEor		sta	seed
			rts									; return												
; ------------------------------------------------------------------------------
joyVal	fcb	$00		
contrl	fcb	$01		
clock		fcb	$00		
sndTyp	fcb	$00		
cocoFlg	fcb	$00		
scanOff	fcb	$00		
seed		fcb	$00		
vByte		fcb	$00		
; ----------------------
reg_B		fcb	$00
reg_D		fcb	$00
reg_E		fcb	$00
; ------------------------------------------------------------------------------------------------------------------------------------------
PgmEnd	equ	*
